@@ -8,6 +8,7 @@
 #include <time.h>
 #include <syslog.h>
 #include "libnagios.h"
+#include "../include/config.h"
 
 #define MSG_DELIM "\1\0\0" /**< message limiter - note this ends up being
 			     \1\0\0\0 on the wire as "" strings null-terminate */
@@ -22,6 +23,7 @@ struct execution_information {
 	struct timeval start;
 	struct timeval stop;
 	float runtime;
+	/* 5DEPR: rusage is deprecated for Nagios, will be removed in 5.0.0 */
 	struct rusage rusage;
 };
 
@@ -178,8 +180,7 @@ int worker_send_kvvec(int sd, struct kvvec *kvv)
 		return -1;
 
 	/* bufsize, not buflen, as it gets us the delimiter */
-	/* ret = write(sd, kvvb->buf, kvvb->bufsize); */
-    ret = nwrite(sd, kvvb->buf, kvvb->bufsize,NULL);
+	ret = nwrite(sd, kvvb->buf, kvvb->bufsize, NULL);
 	free(kvvb->buf);
 	free(kvvb);
 
@@ -214,7 +215,7 @@ int worker_buf2kvvec_prealloc(struct kvvec *kvv, char *buf, unsigned long len, i
 	} while (0)
 
 /* forward declaration */
-static void gather_output(child_process *cp, iobuf *io, int final);
+static int gather_output(child_process *cp, iobuf *io, int final);
 
 static void destroy_job(child_process *cp)
 {
@@ -257,16 +258,23 @@ static void destroy_job(child_process *cp)
 int finish_job(child_process *cp, int reason)
 {
 	static struct kvvec resp = KVVEC_INITIALIZER;
-	struct rusage *ru = &cp->ei->rusage;
-	int i, ret;
+	int i, ret, rd;
 
 	/* get rid of still open filedescriptors */
 	if (cp->outstd.fd != -1) {
-		gather_output(cp, &cp->outstd, 1);
+
+		rd = 1;
+		while(rd > 0) {
+			rd = gather_output(cp, &cp->outstd, 0);
+		}
 		iobroker_close(iobs, cp->outstd.fd);
 	}
 	if (cp->outerr.fd != -1) {
-		gather_output(cp, &cp->outerr, 1);
+
+		rd = 1;
+		while(rd > 0) {
+			rd = gather_output(cp, &cp->outerr, 0);
+		}
 		iobroker_close(iobs, cp->outerr.fd);
 	}
 
@@ -308,14 +316,8 @@ int finish_job(child_process *cp, int reason)
 	kvvec_add_tv(&resp, "stop", cp->ei->stop);
 	kvvec_addkv(&resp, "runtime", mkstr("%f", cp->ei->runtime));
 	if (!reason) {
-		/* child exited nicely (or with a signal, so check wait_status) */
+		/* child exited nicely */
 		kvvec_addkv(&resp, "exited_ok", "1");
-		kvvec_add_tv(&resp, "ru_utime", ru->ru_utime);
-		kvvec_add_tv(&resp, "ru_stime", ru->ru_stime);
-		kvvec_add_long(&resp, "ru_minflt", ru->ru_minflt);
-		kvvec_add_long(&resp, "ru_majflt", ru->ru_majflt);
-		kvvec_add_long(&resp, "ru_inblock", ru->ru_inblock);
-		kvvec_add_long(&resp, "ru_oublock", ru->ru_oublock);
 	} else {
 		/* some error happened */
 		kvvec_addkv(&resp, "exited_ok", "0");
@@ -332,7 +334,8 @@ int finish_job(child_process *cp, int reason)
 
 static int check_completion(child_process *cp, int flags)
 {
-	int result, status;
+	int status;
+	pid_t result;
 
 	if (!cp || !cp->ei->pid) {
 		return 0;
@@ -344,7 +347,7 @@ static int check_completion(child_process *cp, int flags)
 	 */
 	do {
 		errno = 0;
-		result = wait4(cp->ei->pid, &status, flags, &cp->ei->rusage);
+		result = waitpid(cp->ei->pid, &status, flags);
 	} while (result < 0 && errno == EINTR);
 
 	if (result == cp->ei->pid || (result < 0 && errno == ECHILD)) {
@@ -375,7 +378,8 @@ static int check_completion(child_process *cp, int flags)
  */
 static void kill_job(child_process *cp, int reason)
 {
-	int ret, status, reaped = 0;
+	pid_t ret;
+	int status, reaped = 0;
 	int pid = cp ? cp->ei->pid : 0;
 
 	/*
@@ -425,7 +429,7 @@ static void kill_job(child_process *cp, int reason)
 				tv.tv_sec += 5;
 				wlog("Failed to reap child with pid %ld. Next attempt @ %llu.%lu", (long)cp->ei->pid, (unsigned long long)tv.tv_sec, (unsigned long)tv.tv_usec);
 			} else {
-				tv.tv_usec = 250000;
+				tv.tv_usec += 250000;
 				if (tv.tv_usec > 1000000) {
 					tv.tv_usec -= 1000000;
 					tv.tv_sec += 1;
@@ -446,13 +450,13 @@ static void kill_job(child_process *cp, int reason)
 	destroy_job(cp);
 }
 
-static void gather_output(child_process *cp, iobuf *io, int final)
+static int gather_output(child_process *cp, iobuf *io, int final)
 {
 	int retry = 5;
+	int rd;
 
 	for (;;) {
 		char buf[4096];
-		int rd;
 
 		rd = read(io->fd, buf, sizeof(buf));
 		if (rd < 0) {
@@ -488,13 +492,13 @@ static void gather_output(child_process *cp, iobuf *io, int final)
 		if (rd <= 0 || final) {
 			iobroker_close(iobs, io->fd);
 			io->fd = -1;
-			if (!final)
-				check_completion(cp, WNOHANG);
 			break;
 		}
 
 		break;
 	}
+
+	return rd;
 }
 
 
@@ -522,8 +526,7 @@ static void reap_jobs(void)
 	int reaped = 0;
 	do {
 		int pid, status;
-		struct rusage ru;
-		pid = wait3(&status, WNOHANG, &ru);
+		pid = waitpid(-1, &status, WNOHANG);
 		if (pid > 0) {
 			struct child_process *cp;
 
@@ -533,7 +536,6 @@ static void reap_jobs(void)
 				continue;
 			}
 			cp->ret = status;
-			memcpy(&cp->ei->rusage, &ru, sizeof(ru));
 			reaped++;
 			if (cp->ei->state != ESTALE)
 				finish_job(cp, cp->ei->state);
